<?php

namespace Drupal\domain_simple_sitemap;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\domain\DomainInterface;
use Drupal\domain\DomainValidatorInterface;
use Drupal\simple_sitemap\Simplesitemap;

/**
 * The service for domain_simple_sitemap module.
 *
 * @package Drupal\domain_simple_sitemap
 */
class DomainSitemapManager {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The simple sitemap generator.
   *
   * @var \Drupal\simple_sitemap\Simplesitemap
   */
  protected $simpleSitemapManager;

  /**
   * The domain validator.
   *
   * @var \Drupal\domain\DomainValidatorInterface
   */
  protected $domainValidator;

  /**
   * DomainSitemapManager constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\simple_sitemap\Simplesitemap $simpleSitemapManager
   *   The simple sitemap manager.
   * @param \Drupal\domain\DomainValidatorInterface $domainValidator
   *   The domain validator service.
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    Simplesitemap $simpleSitemapManager,
    DomainValidatorInterface $domainValidator
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->simpleSitemapManager = $simpleSitemapManager;
    $this->domainValidator = $domainValidator;
  }

  /**
   * Add sitemap variant for domain.
   *
   * @param \Drupal\domain\DomainInterface $domain
   *   The domain to validate for syntax and uniqueness.
   */
  public function addSitemapVariant(DomainInterface $domain) {
    // Create an sitemap variant.
    $manager = $this->simpleSitemapManager->getSitemapManager();
    $manager->addSitemapVariant($domain->id(), [
      'type' => 'domain',
      'label' => $domain->label(),
    ]);
    // Generate the site map.
    $generator = $manager->getSitemapGenerator('default');
    $generator->setSitemapVariant($domain->id());
    $generator->setSettings(['excluded_languages' => [], 'xsl' => TRUE]);
    if ($domain->status()) {
      $generator->generate([]);
      $generator->publish();
    }
  }

  /**
   * Delete sitemap variant for domain.
   *
   * @param \Drupal\domain\DomainInterface $domain
   *   The domain to validate for syntax and uniqueness.
   */
  public function deleteSitemapVariant(DomainInterface $domain) {
    // Remove the sitemap variant.
    $manager = $this->simpleSitemapManager->getSitemapManager();
    $manager->removeSitemapVariants([$domain->id()]);
    // Remove the sitemap.
    $generator = $manager->getSitemapGenerator('default');
    $generator->setSitemapVariant($domain->id());
    $generator->setSettings(['excluded_languages' => []]);
    $generator->remove();
  }

  /**
   * Checks a domain exists by trying to do an http request to it.
   *
   * @param \Drupal\domain\DomainInterface $domain
   *   The domain to validate for syntax and uniqueness.
   *
   * @return int
   *   The server response code for the request.
   *
   * @see domain_validate()
   */
  private function checkDomain(DomainInterface $domain) {
    $response = $this->domainValidator->checkResponse($domain);
    if ($response >= 200 && $response <= 299) {
      return TRUE;
    }
    return FALSE;
  }

}
