<?php

namespace Drupal\domain_simple_sitemap\Plugin\simple_sitemap\UrlGenerator;

use Drupal\Core\Url;
use Drupal\domain\Entity\Domain;
use Drupal\domain_access\DomainAccessManagerInterface;
use Drupal\simple_sitemap\Plugin\simple_sitemap\UrlGenerator\EntityUrlGenerator;

/**
 * Generates URLs for entity bundles and bundle overrides.
 *
 * @UrlGenerator(
 *   id = "domain_entity",
 *   label = @Translation("Domain entity URL generator"),
 *   description = @Translation("Generates URLs for entity bundles and bundle
 *   overrides."),
 * )
 */
class DomainEntityUrlGenerator extends EntityUrlGenerator {

  /**
   * {@inheritdoc}
   */
  public function getDataSets() {
    $data_sets = [];
    $sitemap_entity_types = $this->entityHelper->getSupportedEntityTypes();

    foreach ($this->generator
      ->setVariants($this->sitemapVariant)
      ->getBundleSettings() as $entity_type_name => $bundles) {
      if (isset($sitemap_entity_types[$entity_type_name])) {
        // Skip this entity type
        // if another plugin is written to override its generation.
        foreach ($this->urlGeneratorManager->getDefinitions() as $plugin) {
          if (isset($plugin['settings']['overrides_entity_type'])
            && $plugin['settings']['overrides_entity_type'] === $entity_type_name) {
            continue 2;
          }
        }

        $entityTypeStorage = $this->entityTypeManager->getStorage($entity_type_name);
        $keys = $sitemap_entity_types[$entity_type_name]->getKeys();

        foreach ($bundles as $bundle_name => $bundle_settings) {
          // Skip if "Content type" is excluded for selected Variant
          // (Index entities of this type in /admin/structure/types/manage/page)
          if (!empty($bundle_settings['index'])) {
            $query = $entityTypeStorage->getQuery();

            if (empty($keys['id'])) {
              $query->sort($keys['id'], 'ASC');
            }
            if (!empty($keys['bundle'])) {
              $query->condition($keys['bundle'], $bundle_name);
            }
            if (!empty($keys['status'])) {
              $query->condition($keys['status'], 1);
            }

            $activeId = \Drupal::service('domain.negotiator')->getActiveId();
            $domainSource = "field_domain_source";

            if (!empty($keys['bundle'])) {
              if ($keys['bundle'] == 'type' && $entity_type_name == 'node') {
                $source_only = \Drupal::config('domain_simple_sitemap.settings')
                  ->get('domain_simple_sitemap_filter');
                if ($source_only) {
                  // Filter by Node Domain source.
                  $query->condition($domainSource . '.target_id', $activeId);
                }
                else {
                  // Filtered by Node Domain access.
                  $orGroupDomain = $query->orConditionGroup()
                    ->condition(DomainAccessManagerInterface::DOMAIN_ACCESS_FIELD . '.target_id', $activeId)
                    ->condition(DomainAccessManagerInterface::DOMAIN_ACCESS_ALL_FIELD, 1);
                  $query->condition($orGroupDomain);
                }
                foreach ($query->execute() as $entity_id) {
                  $data_sets[] = [
                    'entity_type' => $entity_type_name,
                    'id' => $entity_id,
                    'domain_source' => $activeId,
                  ];
                }
              }
              elseif ($keys['bundle'] == 'vid' && $entity_type_name == 'taxonomy_term') {
                // For taxonomy.
                foreach ($query->execute() as $entity_id) {
                  $data_sets[] = [
                    'entity_type' => $entity_type_name,
                    'id' => $entity_id,
                    'domain_source' => $activeId,
                  ];
                }
              }
              else {
                // For all other bundles.
                foreach ($query->execute() as $entity_id) {
                  $data_sets[] = [
                    'entity_type' => $entity_type_name,
                    'id' => $entity_id,
                    'domain_source' => $activeId,
                  ];
                }
              }
            }
          }
        }
      }
    }
    return $data_sets;
  }

  /**
   * {@inheritdoc}
   */
  protected function processDataSet($data_set) {
    // Equal $entity->id().
    $entity_id = $data_set['id'];
    // Equal $entity->getEntityTypeId().
    $entity_type_name = $data_set['entity_type'];
    $entity_source = $data_set['domain_source'];

    if (empty($entity = $this->entityTypeManager->getStorage($entity_type_name)
      ->load($entity_id))) {
      return FALSE;
    }

    // Remove Entity of other sources.
    if ($this->sitemapVariant != $entity_source) {
      return FALSE;
    }

    $entity_settings = $this->generator
      ->setVariants($this->sitemapVariant)
      ->getEntityInstanceSettings($entity_type_name, $entity_id);

    if (empty($entity_settings['index'])) {
      return FALSE;
    }

    // Domain & URL variables.
    $url_object = $entity->toUrl();
    $url_object->setOption('absolute', TRUE);

    // Do not include external paths.
    if (!$url_object->isRouted()) {
      return FALSE;
    }

    $path = $url_object->getInternalPath();
    $url_object->setOption('absolute', TRUE);
    // Photos.
    $url_photo_formatted = "";
    if (!empty($entity_settings['include_images'])) {
      $url_photo = $this->getImages($this->sitemapVariant, $entity_type_name, $entity_id);
      if (!empty($url_photo)) {
        $url_photo_formatted = Url::fromUri($url_photo);
      }
    }

    $paths = [];
    $paths[] = [
      'url' => $url_object,
      'lastmod' => method_exists($entity, 'getChangedTime') ? date('c', $entity->getChangedTime()) : NULL,
      'priority' => isset($entity_settings['priority']) ? $entity_settings['priority'] : NULL,
      'changefreq' => !empty($entity_settings['changefreq']) ? $entity_settings['changefreq'] : NULL,
      'images' => !empty($url_photo_formatted) ? $url_photo_formatted : [],
      // Additional info useful in hooks.
      'meta' => [
        'path' => $path,
        'entity_info' => [
          'entity_type' => $entity_type_name,
          'id' => $entity_id,
        ],
      ],
    ];
    return $paths;
  }

  /**
   * Set Sitemap Variant for specific domain.
   *
   * @param string $sitemap_variant
   *   The sitemap variant.
   *
   * @return $this
   */
  public function setSitemapVariant($sitemap_variant) {
    parent::setSitemapVariant($sitemap_variant);
    $domain = Domain::load($sitemap_variant);
    \Drupal::service('domain.negotiator')->setActiveDomain($domain);
    return $this;
  }

  /**
   * Get images field from entity's fields.
   *
   * @param string $current_variant
   *   The current sitemap variant.
   * @param string $entity_type_name
   *   The entity name of type.
   * @param mixed $entity_id
   *   The Entity ID.
   *
   * @return string
   *   A string containing a URL that may be used to access the image file.
   */
  protected function getImages($current_variant, $entity_type_name, $entity_id) {
    $term_obj = $this->entityTypeManager->getStorage($entity_type_name)
      ->load($entity_id);
    // TODO : change "field_photo" by something not specific.
    if (empty($term_obj)) {
      return "";
    }
    if ($term_obj->hasField('field_image') && isset($term_obj->get('field_image')->entity)) {
      return file_create_url($term_obj->get('field_image')->entity->getFileUri());
    }
    elseif ($term_obj->hasField('field_photo') && isset($term_obj->get('field_photo')->entity)) {
      return file_create_url($term_obj->get('field_photo')->entity->getFileUri());
    }
    elseif ($term_obj->hasField('field_img') && isset($term_obj->get('field_img')->entity)) {
      return file_create_url($term_obj->get('field_img')->entity->getFileUri());
    }
  }

  /**
   * To replace base URL by custom one.
   *
   * @param string $url
   *   The custom URL.
   *
   * @return string
   *   A replaced URL.
   */
  protected function replaceBaseUrlWithCustom($url) {
    $domain = \Drupal::service('domain.negotiator')->getActiveDomain();
    $url_parts = explode("/", $url);
    /*
     * Add a check for the country path module
     * @see https://www.drupal.org/project/domain_simple_sitemap/issues/2922447
     * @see https://www.drupal.org/project/simple_sitemap/issues/3025960
     */
    $domain_suffix = $domain->getThirdPartySetting('country_path', 'domain_path');
    if ($domain_suffix !== NULL) {
      // Need to look at all domains and check each suffix.
      // If this process (generation) was triggered under one of
      // the domains with a country path suffix then we are dealing
      // with $url in the format $baseurl/$current_suffix/$url and need to
      // remove $current_suffix.
      $domain_storage = \Drupal::entityTypeManager()->getStorage('domain');
      $all_domains = $domain_storage->loadMultipleSorted();
      foreach ($all_domains as $current_domain) {
        $cur_suffix = $current_domain->getThirdPartySetting('country_path', 'domain_path');
        if ($cur_suffix != '' && in_array($cur_suffix, $url_parts)) {
          $url = str_replace("/" . $cur_suffix . "/", "/", $url);
          break;
        }
      }
      $custom_url = str_replace($GLOBALS['base_url'], $domain->getScheme() . $domain->getCanonical() . '/' . $domain_suffix, $url);
    }
    else {
      $custom_url = str_replace($GLOBALS['base_url'], $domain->getScheme() . $domain->getCanonical(), $url);
    }
    return $custom_url;
  }

}
